@@ -4,8 +4,10 @@ use 5.010;
 use strict;
 use warnings;
 use Log::Any '$log';
+use Module::Patch 0.07 qw(patch_package);
+use SHARYANTO::Package::Util qw(package_exists);
 
-our $VERSION = '0.05'; # VERSION
+our $VERSION = '0.06'; # VERSION
 
 #use Sub::Uplevel;
 
@@ -14,20 +16,6 @@ require Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT_OK = qw(add_logging_to_package);
 
-# XXX copied from SHARYANTO::Package::Util
-sub package_exists {
-    no strict 'refs';
-
-    my $pkg = shift;
-
-    return unless $pkg =~ /\A\w+(::\w+)*\z/;
-    if ($pkg =~ s/::(\w+)\z//) {
-        return !!${$pkg . "::"}{$1 . "::"};
-    } else {
-        return !!$::{$pkg . "::"};
-    }
-}
-
 sub _default_precall_logger {
     my $args = shift;
     #uplevel 2, $args->{orig}, @{$args->{args}};
@@ -97,7 +85,7 @@ _
         },
         filter_subs => {
             summary => 'Filter subroutines to add logging to',
-            schema => ['any*' => {of=>['regex*', 'code*']}],
+            schema => 'regex*',
             description => <<'_',
 
 The default is to add logging to all non-private subroutines. Private
@@ -109,77 +97,37 @@ _
     result_naked => 1,
 };
 sub add_logging_to_package {
-
     my %args = @_;
 
-    my $packages = $args{packages} or die "Please specify 'packages'";
-    $packages = [$packages] unless ref($packages) eq 'ARRAY';
+    patch_package(
+        $args{packages},
+        [{
+            action => 'wrap',
+            sub_name => ($args{filter_subs} // ':public'),
+            code => sub {
+                my $ctx  = shift;
+                my $orig = shift;
 
-    my $filter = $args{filter_subs} // qr/[^_]/;
-
-    for my $package (@$packages) {
-
-        die "Invalid package name $package"
-            unless $package =~ /\A\w+(::\w+)*\z/;
-
-        # require module
-        unless (package_exists($package)) {
-            eval "use $package; 1" or die "Can't load $package: $@";
-        }
-
-        my $src;
-        # get the calling package symbol table name
-        {
-            no strict 'refs';
-            $src = \%{ $package . '::' };
-        }
-
-        # loop through all symbols in calling package, looking for subs
-        for my $symbol (keys %$src) {
-            # get all code references, make sure they're valid
-            my $sub = *{ $src->{$symbol} }{CODE};
-            next unless defined $sub and defined &$sub;
-
-            my $name = "${package}::$symbol";
-            if (ref($filter) eq 'CODE') {
-                next unless $filter->($name);
-            } else {
-                next unless $name =~ $filter;
-            }
-
-            # save all other slots of the typeglob
-            my @slots;
-
-            for my $slot (qw( SCALAR ARRAY HASH IO FORMAT )) {
-                my $elem = *{ $src->{$symbol} }{$slot};
-                next unless defined $elem;
-                push @slots, $elem;
-            }
-
-            # clear out the source glob
-            undef $src->{$symbol};
-
-            # replace the sub in the source
-            $src->{$symbol} = sub {
-                my $logger;
                 my @args = @_;
                 my %largs = (
-                    orig   => $sub,
-                    name   => $name,
+                    orig   => $orig,
+                    name   => $ctx->{orig_name},
                     args   => [@args],
                 );
 
+                my $logger;
+
                 $logger = $args{precall_logger} // \&_default_precall_logger;
                 $logger->(\%largs);
 
                 my $wa = wantarray;
                 my @res;
                 if ($wa) {
-                    @res =  $sub->(@args);
+                    @res =  $orig->(@args);
                 } elsif (defined $wa) {
-                    $res[0] = $sub->(@args);
+                    $res[0] = $orig->(@args);
                 } else {
-                    $sub->(@args);
+                    $orig->(@args);
                 }
 
                 $logger = $args{postcall_logger} // \&_default_postcall_logger;
@@ -193,17 +141,9 @@ sub add_logging_to_package {
                 } else {
                     return;
                 }
-            };
-
-            # replace the other slot elements
-            for my $elem (@slots) {
-                $src->{$symbol} = $elem;
-            }
-        } # for $symbol
-
-    } # for $package
-
-    1;
+            },
+        }]
+    );
 }
 
 1;
@@ -219,17 +159,21 @@ Log::Any::For::Package - Add logging to package
 
 =head1 VERSION
 
-version 0.05
+version 0.06
 
 =head1 SYNOPSIS
 
+ use My::Module;
+ use My::Other::Module;
  use Log::Any::For::Package qw(add_logging_to_package);
- add_logging_to_package(packages => [qw/My::Module My::Other::Module/]);
+
+ my $h = add_logging_to_package(packages => [qw/My::Module My::Other::Module/]);
+
  # now calls to your module functions are logged, by default at level 'trace'
+ My::Module::foo(...);
 
-=head1 CREDITS
-
-Some code portion taken from L<Devel::TraceMethods>.
+ # restore original subroutines
+ undef $h;
 
 =head1 SEE ALSO
 
@@ -245,7 +189,7 @@ This module has L<Rinci> metadata.
 
 None are exported by default, but they are exportable.
 
-=head2 add_logging_to_package(%args) -> [status, msg, result, meta]
+=head2 add_logging_to_package(%args) -> any
 
 Add logging to class.
 
@@ -265,14 +209,14 @@ Arguments ('*' denotes required arguments):
 
 Classes to add logging to.
 
-=item * B<filter_methods>* => I<array>
+=item * B<filter_methods> => I<regex>
 
 Filter methods to add logging to.
 
 The default is to add logging to all non-private methods. Private methods are
 those prefixed by C<_>.
 
-=item * B<postcall_logger>* => I<code>
+=item * B<postcall_logger> => I<code>
 
 Supply custom postcall logger.
 
@@ -283,7 +227,7 @@ fully-qualified method name), C<result> (arrayref, the method result).
 
 You can use this mechanism to customize logging.
 
-=item * B<precall_logger>* => I<code>
+=item * B<precall_logger> => I<code>
 
 Supply custom precall logger.
 
@@ -298,8 +242,6 @@ You can use this mechanism to customize logging.
 
 Return value:
 
-Returns an enveloped result (an array). First element (status) is an integer containing HTTP status code (200 means OK, 4xx caller error, 5xx function error). Second element (msg) is a string containing error message, or 'OK' if status is 200. Third element (result) is optional, the actual result. Fourth element (meta) is called result metadata and is optional, a hash that contains extra information.
-
 =head1 AUTHOR
 
 Steven Haryanto <stevenharyanto@gmail.com>
